# Generated by roxygen2: do not edit by hand

S3method(plot,RoBSA)
S3method(predict,RoBSA)
S3method(print,RoBSA)
S3method(print,summary.RoBSA)
S3method(summary,RoBSA)
S3method(update,RoBSA)
export(RoBSA)
export(RoBSA.get_option)
export(RoBSA.options)
export(calibrate_meta_analytic)
export(calibrate_quartiles)
export(check_RoBSA)
export(check_setup)
export(contr.independent)
export(contr.meandif)
export(contr.orthonormal)
export(diagnostics)
export(diagnostics_autocorrelation)
export(diagnostics_density)
export(diagnostics_trace)
export(exp_aft_density)
export(exp_aft_hazard)
export(exp_aft_log_density)
export(exp_aft_log_hazard)
export(exp_aft_log_survival)
export(exp_aft_mean)
export(exp_aft_p)
export(exp_aft_q)
export(exp_aft_r)
export(exp_aft_sd)
export(exp_aft_survival)
export(extract_flexsurv)
export(gamma_aft_density)
export(gamma_aft_hazard)
export(gamma_aft_log_density)
export(gamma_aft_log_hazard)
export(gamma_aft_log_survival)
export(gamma_aft_mean)
export(gamma_aft_p)
export(gamma_aft_q)
export(gamma_aft_r)
export(gamma_aft_sd)
export(gamma_aft_survival)
export(get_default_prior_aux)
export(get_default_prior_beta_alt)
export(get_default_prior_beta_null)
export(get_default_prior_factor_null)
export(get_default_prior_intercept)
export(is.RoBSA)
export(llogis_aft_density)
export(llogis_aft_hazard)
export(llogis_aft_log_density)
export(llogis_aft_log_hazard)
export(llogis_aft_log_survival)
export(llogis_aft_mean)
export(llogis_aft_p)
export(llogis_aft_q)
export(llogis_aft_r)
export(llogis_aft_sd)
export(llogis_aft_survival)
export(lnorm_aft_density)
export(lnorm_aft_hazard)
export(lnorm_aft_log_density)
export(lnorm_aft_log_hazard)
export(lnorm_aft_log_survival)
export(lnorm_aft_mean)
export(lnorm_aft_p)
export(lnorm_aft_q)
export(lnorm_aft_r)
export(lnorm_aft_sd)
export(lnorm_aft_survival)
export(plot_density)
export(plot_hazard)
export(plot_models)
export(plot_prediction)
export(plot_survival)
export(prior)
export(prior_factor)
export(prior_informed)
export(prior_informed_medicine_names)
export(prior_none)
export(set_autofit_control)
export(set_convergence_checks)
export(weibull_aft_density)
export(weibull_aft_hazard)
export(weibull_aft_log_density)
export(weibull_aft_log_hazard)
export(weibull_aft_log_survival)
export(weibull_aft_mean)
export(weibull_aft_p)
export(weibull_aft_q)
export(weibull_aft_r)
export(weibull_aft_sd)
export(weibull_aft_survival)
importFrom(Rdpack,reprompt)
importFrom(rlang,.data)
importFrom(survival,Surv)
