#' @title RoBSA: Robust Bayesian survival analysis
#'
#' @description Bayesian model-averaged parametric survival analysis
#' with ability to specify informed prior distributions and draw inference with
#' inclusion Bayes factors. See \insertCite{bartos2021informed;textual}{RoBSA}
#' for more details about the methodology.
#'
#'
#' @name RoBSA-package
#' @author František Bartoš \email{f.bartos96@@gmail.com}
#' @keywords package
#' @aliases RoBSA-package RoBSA_package RoBSA.package
#' @docType package
#' @section
#' User guide: See \insertCite{bartos2021informed;textual}{RoBSA},
#' for details regarding the RoBSA methodology.
#'
#' @references \insertAllCited{}
#' @importFrom survival Surv
#' @importFrom Rdpack reprompt
#' @importFrom rlang .data
"_PACKAGE"
